<?php
class Hiconix {
    private $config;
    private $req;
    private $header;
    private $data = array();
    private $total_prods = 0;
    private $prodmap = array();
    private $loaded_prods = 0;
    private $brand = false;
    private $brand_id = 0;
    private $topcat = 0;
    private $section = false;
    private $section_id = 0;
    private $product = false;
    private $prod_id = 0;
    private $attributes = false;
    private $propvalid = array();

    private $pairs = array(); // все комплекты
    private $pairs_prod = array(); // все товары, образующие комплекты
    private $addedPairs = array(); // обработанные комплекты

    public function __construct() {
        $this->config = $this->getConfig();
        $this->req = $this->buildReq();
        $this->header = 'Authorization: Token ' . $this->config['token'];
        if(!CModule::IncludeModule("iblock")) {
            echo 'Модуль iblock не установлен, установите, пожалуйста';
        }
    }

    private function log($code, $pcent = false, $msg = false) {
        if(!file_exists(LOGFILE)) {
            file_put_contents(LOGFILE, '', FILE_APPEND);
        }
        $testlog = json_decode( file_get_contents(LOGFILE), true );
        if(!isset($testlog['code']) || $testlog['code'] == 'ERROR') {
            exit(0);
        }
        $retval = array('code'=>$code);
        if($pcent !== false) $retval['percent'] = $pcent;
        if($msg) $retval['msg'] = $msg;
        file_put_contents( LOGFILE, json_encode($retval, JSON_UNESCAPED_UNICODE) );
        if($code == 'ERROR') {
            exit(0);
        }
    }

    private function getConfig() {
        $retval = array('IBLOCK_ID' => COption::getOptionInt('hixload', 'IBLOCK_ID', false));
        $retval['token'] = COption::getOptionString('hixload', 'token', false);
        $retval['prefixname'] = COption::getOptionString('hixload', 'prefixname', false);
        $retval['update_price'] = COption::getOptionInt('hixload', 'update_price', false);
        $retval['update_seo'] = COption::getOptionInt('hixload', 'update_seo', false);
        $retval['update_desc'] = COption::getOptionInt('hixload', 'update_desc', false);
        $retval['update_cats'] = COption::getOptionInt('hixload', 'update_cats', false);
        $retval['update_images'] = COption::getOptionInt('hixload', 'update_images', false);
        $retval['pricebump'] = (float)COption::getOptionInt('hixload', 'pricebump', '1.000');
        $retval['brands'] = json_decode(COption::getOptionString('hixload', 'brands', ''), true);
        $retval['attributes'] = json_decode(COption::getOptionString('hixload', 'attributes', ''), true);
        return $retval;
    }

    private function buildReq() {
        $retval = array();
        foreach( $this->config['brands'] as $idx=>$brand ) {
            if(empty($brand['load'])) {
                continue;
            }
            $retval[$brand['code']] = array(
                'idx' => $idx,
                'name' => $brand['name'],
                'path'=> 'https://api.hiconix.ru/api/v1/brands?params[brand]=' . $brand['code'],
                'cat' => (int)$brand['category']
            );
        }
        return $retval;
    }

    private function getImage($path, $dest = 'product', $thumb = true) {
        if(!empty($path['url'])) {
            $path = $path['url'];
        }
        $file = explode('/', $path);
        $fname = array_pop($file);
        $prefix = array_pop($file);
        $fname = explode('?', $fname);
        $fname = $fname[0];
        $dir = DIR_IMAGE . '/' . $dest . '/' . $prefix;
        $file = $dir . '/' . $fname;
        $thumbdir = $dir . '/thumb';
        $thumb = $thumbdir . '/' . $fname;
        if($thumb) {
            if(!is_dir($thumbdir)) {
                mkdir($thumbdir, 0777, true);
            }
        } else {
            if(!is_dir($dir)) {
                mkdir($dir, 0777, true);
            }
        }
        if( file_exists($file) ) {
            if( !file_exists($thumb) && $thumb ) {
                CFile::ResizeImageFile( $file, $thumb, array('width' => 300, 'height' => 300) );
            }
            if($thumb) {
                return array( 'img' => CFile::MakeFileArray($file), 'thumb' => CFile::MakeFileArray($thumb) );
            } else {
                return array( 'img' => CFile::MakeFileArray($file), 'thumb' => false );
            }
        }
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($curl, CURLOPT_TIMEOUT, 5);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $imd = curl_exec($curl);
        if (!curl_errno($curl)) {
            if( curl_getinfo($curl, CURLINFO_HTTP_CODE) == 200 ) {
                if($imd) {
                    file_put_contents($file, $imd);
                    curl_close($curl);
                    if($thumb) {
                        CFile::ResizeImageFile( $file, $thumb, array('width' => 300, 'height' => 300) );
                        return array( 'img' => CFile::MakeFileArray($file), 'thumb' => CFile::MakeFileArray($thumb) );
                    } else {
                        return array( 'img' => CFile::MakeFileArray($file), 'thumb' => false );
                    }
                }
            }
        }
        curl_close($curl);
        return false;
    }

    private function curlHandler($url){
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array($this->header));
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $json = curl_exec($curl);
        curl_close($curl);
        return json_decode($json, true);
    }

    private function getData() {
        $this->total_prods = 0;
        if( is_readable(DATAFILE) ) {
            $this->data = include DATAFILE;
            foreach($this->data as $brand) {
                foreach($brand['categories'] as $cat) {
                    $this->total_prods += count($cat['products']);
                }
            }
            return true;
        }
        return false;
    }

    private function loadProdMap() {
        if(!$this->data) {
            if(!$this->getData()) {
                return false;
            }
        }
        if( is_readable(MAPFILE) ) {
            $this->prodmap = include MAPFILE;
        } else {
            $arF = array("IBLOCK_ID" => $this->config['IBLOCK_ID'], 'PROPERTY_SOURCE'=>'HICONIX');
            $arS = array("ID", "IBLOCK_ID", "NAME", "XML_ID");
            $prods_by_xml_id = array();
            $db_prods = CIBlockElement::GetList( array('ID'=>'ASC'), $arF, false, array("nPageSize"=>100000), $arS );
            while( $prod = $db_prods->getNext() ) {
                $prods_by_xml_id[$prod['XML_ID']] = $prod;
            }
            $prodmap = array();
            foreach($this->data as $brand) {
                foreach($brand['categories'] as $cat) {
                    foreach($cat['products'] as $prod) {
                        $tmp = array( 'HIX_NAME' => $prod['name'], 'MAP_ID' => false, 'MAP_NAME' => false );
                        if(isset($prods_by_xml_id[ $prod['id'] ])) {
                            $tmp['MAP_ID'] = (int)$prods_by_xml_id[ $prod['id'] ]['ID'];
                            $tmp['MAP_NAME'] = $prods_by_xml_id[ $prod['id'] ]['NAME'];
                        }
                        $prodmap[ $prod['id'] ] = $tmp;
                    }
                }
            }
            $dfc = '<?php return ' . var_export($prodmap, true) . ';' . PHP_EOL;
            file_put_contents(MAPFILE, $dfc);
            $this->prodmap = $prodmap;
        }
        return true;
    }

    public function getProdMap($data = array()) {
        if( !$this->loadProdMap() ) {
            return false;
        }
        if( isset($data['order']) && strtolower($data['order']) == 'desc' ) {
            uasort($this->prodmap, function($a, $b){
                if($a['HIX_NAME'] < $b['HIX_NAME']) return 1;
                if($a['HIX_NAME'] > $b['HIX_NAME']) return -1;
                return 0;
            });
        } else {
            uasort($this->prodmap, function($a, $b){
                if($a['HIX_NAME'] > $b['HIX_NAME']) return 1;
                if($a['HIX_NAME'] < $b['HIX_NAME']) return -1;
                return 0;
            });
        }
        $fname = false;
        if(!empty($data['filter_name'])) {
            $fname = $data['filter_name'];
        }
        if($fname) {
            $res = array();
            foreach($this->prodmap as $key=>$itm) {
                if( strpos($itm['HIX_NAME'], $fname) !== false ) {
                    $res[$key] = $itm;
                }
            }
        } else {
            $res = $this->prodmap;
        }
        $start = 0;
        if( !empty($data['start']) ) {
            $start = (int)$data['start'];
        }
        $limit = null;
        if( !empty($data['limit']) ) {
            $limit = max( (int)$data['limit'], 5 );
        }
        if($limit || $start) {
            $list = array_slice($res, $start, $limit, true);
        } else {
            $list = $res;
        }
        $total = count($res);
        return array( 'list' => $list, 'total' => $total );
    }

    public function editProdMap($data) {
        $this->getProdMap();
        if( !$this->prodmap || empty($data['HIX_ID']) ) return false;
        if( !isset($this->prodmap[ $data['HIX_ID'] ]) ) return false;
        if( isset($data['MAP_ID']) && isset($data['MAP_NAME']) ) {
            $this->prodmap[ $data['HIX_ID'] ]['MAP_ID'] = $data['MAP_ID'];
            $this->prodmap[ $data['HIX_ID'] ]['MAP_NAME'] = $data['MAP_NAME'];
        } else {
            $this->prodmap[ $data['HIX_ID'] ]['MAP_ID'] = $this->prodmap[ $data['HIX_ID'] ]['MAP_NAME'] = '';
        }
        $dfc = '<?php return ' . var_export($this->prodmap, true) . ';' . PHP_EOL;
        file_put_contents(MAPFILE, $dfc);
        return true;
    }

    public function getProdSearch($search) {
        $arF = array("IBLOCK_ID" => $this->config['IBLOCK_ID'], 'NAME'=>'%' . $search . '%' );
        $arS = array("ID", "IBLOCK_ID", "NAME");
        $db_prods = CIBlockElement::GetList( array('NAME'=>'ASC'), $arF, false, array("nTopCount" => 6), $arS );
        $res = array();
        while( $prod = $db_prods->getNext() ) {
            $res[] = $prod;
        }
        return $res;
    }

    private function prepareFields() {
        $properties = CIBlockProperty::GetList(Array("sort"=>"asc", "name"=>"asc"), Array("ACTIVE"=>"Y", "IBLOCK_ID"=>$this->config['IBLOCK_ID']));
        $found_price = false;
        $found_pricebump = false;
        $found_update_price = false;
        $found_update_images = false;
        $found_update_desc = false;
        $found_update_seo = false;
        $found_currency = false;
        $found_more_photo = false;
        $found_hix_attribs = false;
        $found_source = false;
        while ($prop_fields = $properties->GetNext()){
            if($prop_fields["CODE"] == 'PRICE') $found_price = true;
            if($prop_fields["CODE"] == 'CURRENCY') $found_currency = true;
            if($prop_fields["CODE"] == 'HIX_PRICEBUMP') $found_pricebump = true;
            if($prop_fields["CODE"] == 'HIX_UPDATE_PRICE') $found_update_price = true;
            if($prop_fields["CODE"] == 'HIX_UPDATE_IMAGES') $found_update_images = true;
            if($prop_fields["CODE"] == 'HIX_UPDATE_DESC') $found_update_desc = true;
            if($prop_fields["CODE"] == 'HIX_UPDATE_SEO') $found_update_seo = true;
            if($prop_fields["CODE"] == 'MORE_PHOTO') $found_more_photo = true;
            if($prop_fields["CODE"] == 'HIX_ATTRIBUTES') $found_hix_attribs = (int)$prop_fields["ID"];
            if($prop_fields["CODE"] == 'SOURCE') $found_source = (int)$prop_fields["ID"];
        }
        $ibp = new CIBlockProperty();
        if(!$found_price) {
            $newField = array(
                "NAME" => "Цена",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "PRICE",
                "PROPERTY_TYPE" => "N",
                "IBLOCK_ID" => $this->config['IBLOCK_ID']
            );
            $ibp->Add($newField);
        }
        if(!$found_currency) {
            $newField = array(
                "NAME" => "Валюта",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "CURRENCY",
                "PROPERTY_TYPE" => "S",
                "IBLOCK_ID" => $this->config['IBLOCK_ID']
            );
            $ibp->Add($newField);
        }
        if(!$found_pricebump) {
            $newField = array(
                "NAME" => "Коэффициент наценки",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "HIX_PRICEBUMP",
                "PROPERTY_TYPE" => "S",
                "IBLOCK_ID" => $this->config['IBLOCK_ID']
            );
            $ibp->Add($newField);
        }
        if(!$found_update_price) {
            $newField = array(
                "NAME" => "Обновлять цену",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "HIX_UPDATE_PRICE",
                "PROPERTY_TYPE" => "L",
                "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                "VALUES" => array (
                    array("VALUE" => "По умолчанию", "XML_ID"=> "-1", "SORT"=>100, "DEF" => "Y"),
                    array("VALUE" => "Обновлять", "XML_ID"=> "1", "SORT"=>200, "DEF" => "N"),
                    array("VALUE" => "Не обновлять", "XML_ID"=> "2", "SORT"=>300, "DEF" => "N")
                )
            );
            $ibp->Add($newField);
        }
        if(!$found_update_images) {
            $newField = array(
                "NAME" => "Обновлять изображения",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "HIX_UPDATE_IMAGES",
                "PROPERTY_TYPE" => "L",
                "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                "VALUES" => array (
                    array("VALUE" => "По умолчанию", "XML_ID"=> "-1", "SORT"=>100, "DEF" => "Y"),
                    array("VALUE" => "Обновлять", "XML_ID"=> "1", "SORT"=>200, "DEF" => "N"),
                    array("VALUE" => "Не обновлять", "XML_ID"=> "2", "SORT"=>300, "DEF" => "N")
                )
            );
            $ibp->Add($newField);
        }
        if(!$found_update_desc) {
            $newField = array(
                "NAME" => "Обновлять описания",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "HIX_UPDATE_DESC",
                "PROPERTY_TYPE" => "L",
                "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                "VALUES" => array (
                    array("VALUE" => "По умолчанию", "XML_ID"=> "-1", "SORT"=>100, "DEF" => "Y"),
                    array("VALUE" => "Обновлять", "XML_ID"=> "1", "SORT"=>200, "DEF" => "N"),
                    array("VALUE" => "Не обновлять", "XML_ID"=> "2", "SORT"=>300, "DEF" => "N")
                )
            );
            $ibp->Add($newField);
        }
        if(!$found_update_seo) {
            $newField = array(
                "NAME" => "Обновлять ЧПУ",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "HIX_UPDATE_SEO",
                "PROPERTY_TYPE" => "L",
                "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                "VALUES" => array (
                    array("VALUE" => "По умолчанию", "XML_ID"=> "-1", "SORT"=>100, "DEF" => "Y"),
                    array("VALUE" => "Обновлять", "XML_ID"=> "1", "SORT"=>200, "DEF" => "N"),
                    array("VALUE" => "Не обновлять", "XML_ID"=> "2", "SORT"=>300, "DEF" => "N")
                )
            );
            $ibp->Add($newField);
        }

        // ->>> Сохраняем значения ID вариантов полей типа "Список"
        $tmp = array();
        $prenum_db = CIBlockProperty::GetPropertyEnum(
            'HIX_UPDATE_PRICE',
            array("ID"=>"ASC"),
            array("IBLOCK_ID" => $this->config['IBLOCK_ID'])
        );
        while( $pr = $prenum_db->getNext() ) {
            $tmp[ $pr['XML_ID'] ] = $pr;
        }
        $this->propvalid['HIX_UPDATE_PRICE'] = $tmp;

        $tmp = array();
        $prenum_db = CIBlockProperty::GetPropertyEnum(
            'HIX_UPDATE_IMAGES',
            array("ID"=>"ASC"),
            array("IBLOCK_ID" => $this->config['IBLOCK_ID'])
        );
        while( $pr = $prenum_db->getNext() ) {
            $tmp[ $pr['XML_ID'] ] = $pr;
        }
        $this->propvalid['HIX_UPDATE_IMAGES'] = $tmp;

        $tmp = array();
        $prenum_db = CIBlockProperty::GetPropertyEnum(
            'HIX_UPDATE_DESC',
            array("ID"=>"ASC"),
            array("IBLOCK_ID" => $this->config['IBLOCK_ID'])
        );
        while( $pr = $prenum_db->getNext() ) {
            $tmp[ $pr['XML_ID'] ] = $pr;
        }
        $this->propvalid['HIX_UPDATE_DESC'] = $tmp;

        $tmp = array();
        $prenum_db = CIBlockProperty::GetPropertyEnum(
            'HIX_UPDATE_SEO',
            array("ID"=>"ASC"),
            array("IBLOCK_ID" => $this->config['IBLOCK_ID'])
        );
        while( $pr = $prenum_db->getNext() ) {
            $tmp[ $pr['XML_ID'] ] = $pr;
        }
        $this->propvalid['HIX_UPDATE_SEO'] = $tmp;
        // ->>>

        if(!$found_more_photo) {
            $newField = array(
                "NAME" => "Картинки",
                "ACTIVE" => "Y",
                "SORT" => "400",
                "CODE" => "MORE_PHOTO",
                "PROPERTY_TYPE" => "F",
                "MULTIPLE" => "Y",
                "IBLOCK_ID" => $this->config['IBLOCK_ID']
            );
            $ibp->Add($newField);
        }
        if(!$found_source) {
            $newField = array(
                "NAME" => "Источник",
                "ACTIVE" => "Y",
                "SORT" => "1200",
                "CODE" => "SOURCE",
                "PROPERTY_TYPE" => "S",
                "MULTIPLE" => "N",
                "IBLOCK_ID" => $this->config['IBLOCK_ID']
            );
            $res = $ibp->Add($newField);
        }
        if(!$this->config['attributes']['default_field']) {
            if($found_hix_attribs) {
                $this->config['attributes']['default_field'] = $found_hix_attribs;
            } else {
                $newField = array(
                    "NAME" => "Характеристики",
                    "ACTIVE" => "Y",
                    "SORT" => "400",
                    "CODE" => "HIX_ATTRIBUTES",
                    "PROPERTY_TYPE" => "S",
                    "MULTIPLE" => "Y",
                    "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                    'WITH_DESCRIPTION' => 'Y',
                    'MULTIPLE_CNT' => 1
                );
                $found_hix_attribs = $ibp->Add($newField);
                $this->config['attributes']['default_field'] = $found_hix_attribs;
            }
        }
        // Свойство секции-источник для пометки секций, созданных при загрузке
        $rsData = CUserTypeEntity::GetList(
            array($by=>$order),
            array('ENTITY_ID' => 'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION', 'FIELD_NAME' => 'UF_SOURCE')
        );
        if( !$rsData->Fetch() ) {
            $ent = new CUserTypeEntity();
            $aUserFields    = array(
                'ENTITY_ID'         => 'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION',
                'FIELD_NAME'        => 'UF_SOURCE',
                'USER_TYPE_ID'      => 'string',
                'SORT'              => 500,
                'MULTIPLE'          => 'N',
                'MANDATORY'         => 'N',
                'SHOW_FILTER'       => 'N',
                'SHOW_IN_LIST'      => '',
                'EDIT_IN_LIST'      => '',
                'IS_SEARCHABLE'     => 'Y',
                'SETTINGS'          => array(
                    'DEFAULT_VALUE' => '',
                    'SIZE'          => '16',
                    'ROWS'          => '1',
                    'MIN_LENGTH'    => '0',
                    'MAX_LENGTH'    => '0',
                    'REGEXP'        => ''
                ),
                'EDIT_FORM_LABEL'   => array(
                    'ru'    => 'Источник',
                    'en'    => 'Source'
                ),
                'LIST_COLUMN_LABEL' => array(
                    'ru'    => 'Пользовательское свойство',
                    'en'    => 'User field'
                ),
                'LIST_FILTER_LABEL' => array(
                    'ru'    => 'Пользовательское свойство',
                    'en'    => 'User field'
                ),
                'HELP_MESSAGE'      => array(
                    'ru'    => '',
                    'en'    => ''
                )
            );
            $iUserFieldId   = $ent->Add( $aUserFields );
        }
        // Свойство "Заголовок окна браузера" (в минимальной поставке объявлено для инфоблока "1С каталог")
        $rsData = CUserTypeEntity::GetList(
            array($by=>$order),
            array('ENTITY_ID' => 'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION', 'FIELD_NAME' => 'UF_BROWSER_TITLE')
        );
        if( !$rsData->Fetch() ) {
            if(!isset($ent) ) {
                $ent = new CUserTypeEntity();
            }
            $aUserFields    = array(
                'ENTITY_ID'         => 'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION',
                'FIELD_NAME'        => 'UF_BROWSER_TITLE',
                'USER_TYPE_ID'      => 'string',
                'SORT'              => 500,
                'MULTIPLE'          => 'N',
                'MANDATORY'         => 'N',
                'SHOW_FILTER'       => 'N',
                'SHOW_IN_LIST'      => '',
                'EDIT_IN_LIST'      => '',
                'IS_SEARCHABLE'     => 'Y',
                'SETTINGS'          => array(
                    'DEFAULT_VALUE' => '',
                    'SIZE'          => '20',
                    'ROWS'          => '1',
                    'MIN_LENGTH'    => '0',
                    'MAX_LENGTH'    => '0',
                    'REGEXP'        => ''
                ),
                'EDIT_FORM_LABEL'   => array(
                    'ru'    => 'Заголовок окна браузера',
                    'en'    => 'Browser title'
                ),
                'LIST_COLUMN_LABEL' => array(
                    'ru'    => 'Заголовок окна браузера',
                    'en'    => 'Browser title'
                ),
                'LIST_FILTER_LABEL' => array(
                    'ru'    => 'Заголовок окна браузера',
                    'en'    => 'Browser title'
                ),
                'HELP_MESSAGE'      => array(
                    'ru'    => '',
                    'en'    => ''
                )
            );
            $iUserFieldId   = $ent->Add( $aUserFields );
        }
    }

    private function getSection($is_brand = false) {
        global $USER_FIELD_MANAGER;
        // Загрузка бренда
        if($is_brand) {
            if($this->config['brands'][$this->brand_id]['category']) {
                return $this->config['brands'][$this->brand_id]['category'];
            }
            $pic = $this->getImage($this->brand['logo'], 'brand');
            $arF = array('IBLOCK_ID'=>$this->config['IBLOCK_ID'], 'UF_SOURCE'=>'HICONIX', 'XML_ID' => $this->brand['id']);
            $arS = array( 'ID' );
            $db_sect = CIBlockSection::GetList( array(), $arF, false, $arS );
            if($topcat = $db_sect->getNextElement()) {
                $topcat = $topcat->getFields();
                $arU = array();
                if($this->config['update_seo']) {
                    $arU['CODE'] = $this->data[$this->brand_id]['slug'];
                }
                if($this->config['update_cats']){
                    $arU['DESCRIPTION'] = $this->brand['description'];
                    $arU['DESCRIPTION_TYPE'] = 'html';
                    $arU['DETAIL_PICTURE'] = $pic['img'];
                    $arU['PICTURE'] = $pic['thumb'];
                }
                if($arU) {
                    $bs = new CIBlockSection();
                    if( !$bs->Update( $topcat['ID'], $arU) ) {
                        $this->log('ERROR', false, $bs->LAST_ERROR );
                        return 0;
                    }
                }
                $this->config['brands'][$this->brand_id]['category'] = (int)$topcat['ID'];
                return (int)$topcat['ID'];
            }
            $arA = Array(
                "ACTIVE" => 'Y',
                "IBLOCK_SECTION_ID" => 0,
                "IBLOCK_ID" => $this->config['IBLOCK_ID'],
                "NAME" => $this->brand['name'],
                "SORT" => $this->brand['position'],
                "DETAIL_PICTURE" => $pic['img'],
                "PICTURE" => $pic['thumb'],
                "DESCRIPTION" => $this->brand['description'],
                "DESCRIPTION_TYPE" => 'html',
                "CODE" => $this->brand['slug'],
                "XML_ID" => $this->brand['id'],
            );
            $bs = new CIBlockSection();
            if( !$bcid = $bs->Add($arA) ) {
                $this->log('ERROR', false, $bs->LAST_ERROR );
                return 0;
            } else {
                $this->config['brands'][$this->brand_id]['category'] = $bcid;
                $USER_FIELD_MANAGER->Update( 'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION', $bcid, array('UF_SOURCE' => 'HICONIX') );
                return $bcid;
            }
        }
        // Загрузка подкатегории
        if( empty($this->config['brands'][$this->brand_id]['loadcats']) ) {
            return $this->topcat;
        }
        $pic = $this->getImage($this->section['image'], 'section');
        $arF = array('IBLOCK_ID'=>$this->config['IBLOCK_ID'], 'UF_SOURCE'=>'HICONIX', 'XML_ID' => $this->section['id']);
        $arS = array( 'ID' );
        $db_sect = CIBlockSection::GetList( array(), $arF, false, $arS );
        if($section = $db_sect->getNextElement()) {
            $section = $section->getFields();
            $arU = array();
            if($this->config['update_seo']) {
                $arU['CODE'] = $this->section['slug'];
            }
            if($this->config['update_cats']){
                $arU['DESCRIPTION'] = $this->section['description'];
                $arU['DESCRIPTION_TYPE'] = 'html';
                $arU['DETAIL_PICTURE'] = $pic['img'];
                $arU['PICTURE'] = $pic['thumb'];
            }
            if($arU) {
                $bs = new CIBlockSection();
                if( !$bs->Update( $section['ID'], $arU) ) {
                    $this->log('ERROR', false, $bs->LAST_ERROR );
                    return 0;
                }
            }
            if($this->config['update_cats']){
                $USER_FIELD_MANAGER->Update(
                    'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION',
                    (int)$section['ID'],
                    array('UF_BROWSER_TITLE' => $this->section['fullname'])
                );
            }
            return (int)$section['ID'];
        }
        $arA = Array(
            "ACTIVE" => 'Y',
            "IBLOCK_SECTION_ID" => $this->topcat,
            "IBLOCK_ID" => $this->config['IBLOCK_ID'],
            "NAME" => $this->section['name'],
            "SORT" => $this->section['position'],
            "DETAIL_PICTURE" => $pic['img'],
            "PICTURE" => $pic['thumb'],
            "DESCRIPTION" => $this->section['description'],
            "DESCRIPTION_TYPE" => 'html',
            "CODE" => $this->section['slug'],
            "XML_ID" => $this->section['id']
        );
        $bs = new CIBlockSection();
        if( !$bcid = $bs->Add($arA) ) {
            $this->log('ERROR', false, $bs->LAST_ERROR );
            return 0;
        } else {
            $this->config['brands'][$this->brand_id]['category'] = $bcid;
            $USER_FIELD_MANAGER->Update(
                'IBLOCK_' . $this->config['IBLOCK_ID'] . '_SECTION',
                $bcid,
                array('UF_SOURCE' => 'HICONIX', 'UF_BROWSER_TITLE' => $this->section['fullname'])
            );
            return $bcid;
        }
    }

    private function prepareAttrs() {
        $retval = array();
        $pattr = array();
        foreach($this->product['parameters'] as $pa) {
            $code = $pa['param'];
            if($code == 'price') {
                $retval['price'] = array('value'=>$pa['value'] . ' ' . $pa['currency'], 'currency' => $pa['currency']);
                continue;
            }
            unset($pa['param']);
            $pattr[$code] = $pa;
        }
        $pattr['code'] = array('value' => $this->product['hiconixcode']);
        foreach($this->config['attributes'] as $code=>$data) {
            if($code == 'default_field') continue;
            $data['value'] = false;
            switch($code) {
//                case 'length_width_height':
//                    $v = '';
//                    if(isset($pattr['length'])) {
//                        if(isset($pattr['length']['indoor_value'])) {
//                            $v = array(
//                                'indoor'=> 'Размеры внутреннего блока: ' . $pattr['length']['indoor_value'],
//                                'outdoor'=> 'Размеры наружного блока: ' . $pattr['length']['outdoor_value']
//                            );
//                        } elseif(isset($pattr['length']['value'])) {
//                            $v = $pattr['length']['value'];
//                        }
//                    }
//                    if(isset($pattr['width'])) {
//                        if(isset($pattr['width']['indoor_value'])) {
//                            if($v) {
//                                $v['indoor'] .= 'x' . $pattr['width']['indoor_value'];
//                                $v['outdoor'] .= 'x' . $pattr['width']['outdoor_value'];
//                            }
//                        } elseif(isset($pattr['width']['value'])) {
//                            if($v) $v .= 'x' . $pattr['width']['value'];
//                        }
//                    }
//                    if(isset($pattr['height'])) {
//                        if(isset($pattr['height']['indoor_value'])) {
//                            if($v) {
//                                $v['indoor'] .= 'x' . $pattr['height']['indoor_value'];
//                                $v['outdoor'] .= 'x' . $pattr['height']['outdoor_value'];
//                            }
//                        } elseif(isset($pattr['height']['value'])) {
//                            if($v) $v .= 'x' . $pattr['height']['value'];
//                        }
//                    }
//                    if(!empty($data['unit'])) {
//                        if(is_array($v)) {
//                            $v['indoor'] .= ' ' . $data['unit'];
//                            $v['outdoor'] .= ' ' . $data['unit'];
//                            $v = implode('<br>', $v);
//                        } else {
//                            $v .= ' ' . $data['unit'];
//                        }
//                    }
//                    $data['value'] = $v;
//                    break;
//                case 'weight':
//                    $v = '';
//                    if(isset($pattr['weight']['indoor_value'])) {
//                        $v = 'Вес внутреннего блока: ' . $pattr['weight']['indoor_value'] . '[*unit*]<br>' .
//                            'Вес наружного блока: ' . $pattr['weight']['outdoor_value'] . '[*unit*]<br>' .
//                            'Общий вес: ' . $pattr['weight']['value'] . '[*unit*]';
//                    } elseif(isset($pattr['weight']['value'])) {
//                        $v = $pattr['weight']['value'] . '[*unit*]';
//                    }
//                    if(!empty($data['unit'])) {
//                        $unit = ' ' . $data['unit'];
//                    } else {
//                        $unit = '';
//                    }
//                    $data['value'] = str_replace('[*unit*]', $unit, $v);
//                    break;
                case 'country':
                    $v = '';
                    if(isset($pattr['country']['indoor_value'])) {
                        $v = 'Внутренний блок: ' . $pattr['country']['indoor_value'] . '<br>' .
                            'Наружный блок: ' . $pattr['country']['outdoor_value'];
                    } elseif(isset($pattr['country']['value'])) {
                        $v = $pattr['country']['value'];
                    }
                    $data['value'] = $v;
                    break;
                case 'coolpower':
                case 'heatpower':
                case 'heatnetpower':
                case 'coolnetpower':
                case 'gas_tube_diam':
                case 'liquid_tube_diam':
                    $v = '';
                    if(isset($pattr[$code]) && isset($pattr[$code]['value'])) {
                        $v = round((float)$pattr[$code]['value'], 2);
                        if(!empty($pattr[$code]['units'])) {
                            $v .= ' ' . $pattr[$code]['units'];
                        }
                    }
                    $data['value'] = $v;
                    break;
                case 'warranty':
                case 'inverter':
                    if(empty($pattr[$code]['value'])) {
                        $data['value'] = 'Нет';
                    } else {
                        $data['value'] = 'Да';
                    }
                    break;
                default: {
                    if(isset($pattr[$code]) && isset($pattr[$code]['value'])) {
                        $data['value'] = $pattr[$code]['value'];
                        if(!empty($pattr[$code]['units'])) {
                            $data['value'] .= ' ' . $pattr[$code]['units'];
                        }
                        if(!empty($pattr[$code]['description'])) {
                            $data['name'] = $pattr[$code]['description'];
                        }
                    } else {
                        $data['value'] = false;
                    }
                }
            }
            if($data['value'] !== false) {
                if($code == 'price') {
                    $retval[$code] = $data;
                } else {
                    $map = (int)$data['map'];
                    if($map) {
                        $retval[$map] = $data['value'];
                    } else {
                        $df = (int)$this->config['attributes']['default_field'];
                        if(isset($retval[$df])) {
                            $retval[$df][$code] = array('VALUE'=>$data['value'], 'DESCRIPTION'=>$data['name'], 'VALUE_XML_ID' => $code);
                        } else {
                            $retval[$df] = array( $code => array('VALUE'=>$data['value'], 'DESCRIPTION'=>$data['name'], 'VALUE_XML_ID' => $code) );
                        }
                    }
                }
            }
        }
        return $retval;
    }

    private function getMorePhotos($pxs) {
        $retval = array();
        foreach($pxs as $px) {
            $pf = $this->getImage($px, 'product', false);
            $retval[] = array('VALUE'=>$pf['img'], 'DESCRIPTION'=>'');
        }
        return $retval;
    }

    private function getDefAttr($pid = false) {
        $values = array();
        $retval = array();
        $df = (int)$this->config['attributes']['default_field'];
        if($pid) {
            $db_props = CIBlockElement::GetProperty($this->config['IBLOCK_ID'], $pid, array(), array('ID'=>$this->config['attributes']['default_field']));
            while( $res = $db_props->Fetch() ) {
                $values[$res['DESCRIPTION']] = array('VALUE'=>$res['VALUE'], 'DESCRIPTION' => $res['DESCRIPTION']);
            }
            foreach($this->config['attributes'] as $code=>$attr) {
                if($code == 'default_value' || $code == 'price' || $attr['map']) continue;
                if(isset($values[ $attr['name'] ])) {
                    if( !empty($attr['load']) ) {
                        $retval[] = array(
                            'VALUE' => $this->attributes[$df][$code]['VALUE'],
                            'DESCRIPTION' => $this->attributes[$df][$code]['DESCRIPTION'],
                        );
                    } else {
                        $retval[] = array(
                            'VALUE' => $values[ $attr['name'] ]['VALUE'],
                            'DESCRIPTION' => $values[ $attr['name'] ]['DESCRIPTION'],
                        );
                    }
                } elseif(!empty($attr['load'])) {
                    $retval[] = array(
                        'VALUE' => $this->attributes[$df][$code]['VALUE'],
                        'DESCRIPTION' => $this->attributes[$df][$code]['DESCRIPTION'],
                    );
                }
            }
            return $retval;
        }
        foreach($this->config['attributes'] as $code=>$attr) {
            if($code == 'default_value' || $code == 'price' || $attr['map']) continue;
            $retval[] = array(
                'VALUE' => $this->attributes[$df][$code]['VALUE'],
                'DESCRIPTION' => $this->attributes[$df][$code]['DESCRIPTION'],
            );
        }
        return $retval;
    }

    private function getProd() {
        $df = (int)$this->config['attributes']['default_field'];
        $pics = $this->product['images'];
        $bs = new CIBlockElement();
        usort($pics, function($a, $b){
            if((int)$a['position'] < (int)$b['position']) return -1;
            if((int)$a['position'] > (int)$b['position']) return 1;
            return 0;
        });
        $mainpix = $this->getImage( array_shift($pics), 'product' );
        $morepix = $this->getMorePhotos($pics);
        $pid = false;
        if(!empty($this->prodmap[ $this->product['id'] ]['MAP_ID'])) {
            $pid = $this->prodmap[ $this->product['id'] ]['MAP_ID'];
        }
        if( $pid ) {
            // Получаем свойства объекта
            $props_by_id = array();
            $props_by_code = array();
            $db_props = CIBlockElement::GetProperty( $this->config['IBLOCK_ID'], $pid );
            while( $pr = $db_props->getNext() ) {
                $props_by_id[$pr['ID']] = $pr;
                if(!empty($pr['CODE'])) {
                    $props_by_code[$pr['CODE']] = $pr;
                }
            }
            // Рассчитываем результирующие параметры обновления
            $pricebump = $this->config['pricebump'];
            if(isset($props_by_code['HIX_PRICEBUMP']['VALUE'])) {
                $locval = (float)$props_by_code['HIX_PRICEBUMP']['VALUE'];
                if($locval > 0.01) {
                    $bumpval = $pricebump = $locval;
                }
            } else {
                $bumpval = '';
            }
            $update_seo = $this->config['update_seo']? 1 : 2;
            if( !empty($props_by_code['HIX_UPDATE_SEO']['VALUE_XML_ID']) ) {
                $locval = (int)$props_by_code['HIX_UPDATE_SEO']['VALUE_XML_ID'];
                $upseo_val = (int)$props_by_code['HIX_UPDATE_SEO']['VALUE'];
                if($locval > 0) {
                    $update_seo = $locval;
                }
            } else {
                $upseo_val = '';
            }
            $update_desc = $this->config['update_desc']? 1 : 2;
            if( !empty($props_by_code['HIX_UPDATE_DESC']['VALUE_XML_ID']) ) {
                $locval = (int)$props_by_code['HIX_UPDATE_DESC']['VALUE_XML_ID'];
                $updesc_val = (int)$props_by_code['HIX_UPDATE_DESC']['VALUE'];
                if($locval > 0) {
                    $update_desc = $locval;
                }
            } else {
                $updesc_val = '';
            }
            $update_price = $this->config['update_price']? 1 : 2;
            if( !empty($props_by_code['HIX_UPDATE_PRICE']['VALUE_XML_ID']) ) {
                $locval = (int)$props_by_code['HIX_UPDATE_PRICE']['VALUE_XML_ID'];
                $upprice_val = (int)$props_by_code['HIX_UPDATE_PRICE']['VALUE'];
                if($locval > 0) {
                    $update_price = $locval;
                }
            } else {
                $upprice_val = '';
            }
            $update_images = $this->config['update_images']? 1 : 2;
            if( !empty($props_by_code['HIX_UPDATE_IMAGES']['VALUE_XML_ID']) ) {
                $locval = (int)$props_by_code['HIX_UPDATE_IMAGES']['VALUE_XML_ID'];
                $upimages_val = (int)$props_by_code['HIX_UPDATE_IMAGES']['VALUE'];
                if($locval > 0) {
                    $update_images = $locval;
                }
            } else {
                $upimages_val = '';
            }
            // Обновляем данные
            $arU = array();
            $PROPS = array(
                'SOURCE'=>'HICONIX',
                'HIX_PRICEBUMP' => $bumpval,
                'HIX_UPDATE_SEO' => $upseo_val,
                'HIX_UPDATE_DESC' => $updesc_val,
                'HIX_UPDATE_PRICE' => $upprice_val,
                'HIX_UPDATE_IMAGES' => $upimages_val
            );
            if($update_seo == 1) {
                $arU['CODE'] = $this->product['slug'];
            }
            if($update_desc == 1){
                $arU['DETAIL_TEXT'] = $this->product['description'];
                $arU['DETAIL_TEXT_TYPE'] = 'html';
                $arU['PREVIEW_TEXT'] = $this->product['teaser'];
                $arU['PREVIEW_TEXT_TYPE'] = 'html';
            }
            if($update_images == 1){
                $arU['PREVIEW_PICTURE'] = $mainpix['thumb'];
                $arU['DETAIL_PICTURE'] = $mainpix['img'];
                if($morepix) {
                    $PROPS['MORE_PHOTO'] = $morepics;
                }
            }
            if($update_price == 1){
                $PROPS['PRICE'] = $this->attributes['price']['value'] * $pricebump;
                $PROPS['CURRENCY'] = $this->attributes['price']['currency'];
            } else {
                $PROPS['PRICE'] = $props_by_code['PRICE']['VALUE'] * $pricebump;
                $PROPS['CURRENCY'] = $props_by_code['CURRENCY']['VALUE'];
            }
            unset($this->attributes['price']);
            foreach($this->config['attributes'] as $code=>$attr) {
                if( $code == 'default_field' || empty($attr['map']) ) continue;
                if( !empty($attr['load']) ) {
                    $PROPS[ $attr['map'] ] = $this->attributes[ $attr['map'] ];
                } else {
                    $PROPS[ $attr['map'] ] = $props_by_id[ $attr['map'] ]['VALUE'];
                }
            }
            $PROPS[$df] = $this->getDefAttr($pid);
            $arU['PROPERTY_VALUES'] = $PROPS;
            if($arU) {
                if( !$bs->Update( $pid, $arU) ) {
                    $this->log('ERROR', false, $bs->LAST_ERROR );
                    return 0;
                }
            }
            return (int)$pid;
        }
        $arA = array(
            "IBLOCK_SECTION_ID" => $this->section_id,
            "IBLOCK_ID"      => $this->config['IBLOCK_ID'],
            "NAME"           => $this->product['name'],
            "CODE"           => $this->product['slug'],
            "ACTIVE"         => 'Y',
            "PREVIEW_TEXT"   => $this->product['teaser'],
            "PREVIEW_TEXT_TYPE"   => 'html',
            "PREVIEW_PICTURE"   => $mainpix['thumb'],
            "DETAIL_TEXT"    => $this->product['description'],
            "DETAIL_PICTURE" => $mainpix['img'],
            "XML_ID" => $this->product['id']
        );
        $PROPS = array(
            'SOURCE'=>'HICONIX',
            'PRICE'=>$this->attributes['price']['value'] * $this->config['pricebump'],
            'CURRENCY'=>$this->attributes['price']['currency'],
            'HIX_PRICEBUMP' => '',
            'HIX_UPDATE_SEO' => $this->propvalid['HIX_UPDATE_SEO'][-1],
            'HIX_UPDATE_DESC' => $this->propvalid['HIX_UPDATE_DESC'][-1],
            'HIX_UPDATE_PRICE' => $this->propvalid['HIX_UPDATE_PRICE'][-1],
            'HIX_UPDATE_IMAGES' => $this->propvalid['HIX_UPDATE_IMAGES'][-1]
        );
        unset($this->attributes['price']);
        if($morepix) {
            $PROPS['MORE_PHOTO'] = $morepix;
        }
        foreach($this->config['attributes'] as $code=>$attr) {
            if( $code == 'default_field' || empty($attr['map']) ) continue;
            $PROPS[ $attr['map'] ] = $this->attributes[ $attr['map'] ];
        }
        $PROPS[$df] = $this->getDefAttr($this->product['ID']);
        $arA['PROPERTY_VALUES'] = $PROPS;

        if( !($pid = $bs->Add( $arA )) ) {
            $this->log('ERROR', false, $bs->LAST_ERROR );
            return 0;
        }
        return $pid;
    }

    public function doWrite() {
        $this->loadProdMap();
        $this->prepareFields();
        foreach($this->data as $idx=>&$brand) {
            if( empty($this->config['brands'][$idx]['load']) ) {
                continue;
            }
            $this->brand = &$brand;
            $this->brand_id = $idx;
            $topcat = $this->getSection(true);
            if(!$topcat) {
                //exit(0);
            }
            $this->topcat = $topcat;
            foreach($brand['categories'] as &$sect) {
                $this->section = &$sect;
                $this->section_id = $this->getSection();
                foreach($sect['products'] as $prod) {
                    $this->product = &$prod;
                    $this->attributes = $this->prepareAttrs();
                    $this->prod_id = $this->getProd();
                    $percent = round( 100.0 * (float)(++$this->loaded_prods)/(float)$this->total_prods );
                    $this->log('LOAD', $percent, 'Бренд: ' . $brand['name'] . ', категория: ' . $sect['name'] . ', товар ' . $this->loaded_prods . ' из ' . $this->total_prods);
                }
            }
        }
        $this->log('COMPLETE', false, 'Каталог Hiconix загружен в магазин и обновлен.');
        COption::SetOptionString('hixload', 'brands', json_encode($this->config['brands'], JSON_UNESCAPED_UNICODE));
        COption::SetOptionString('hixload', 'attributes', json_encode($this->config['attributes'], JSON_UNESCAPED_UNICODE));
        unlink(MAPFILE);
        unlink(DATAFILE);
    }

    public function debug() {
        // Отладочный код
    }

    public function doLoad() {
        $this->loadData();
        $this->log('LOAD', false, 'Построение карты отображения товаров каталога на товары в магазине');
        $this->loadProdMap();
        $this->log('LOADED', false, 'Каталог загружен во временный файл. Теперь Вы можете отредактировать карту отображения товаров(если нужно) и запустить загрузку каталога в магазин');
    }

    private function loadData() {
        $this->getPairs();
        $this->getPairsProd();
        $total = count($this->req);
        $bcnt = 0;
        $percent = 0;
        $this->total_prods = 0;
        foreach($this->req as $brand) {
            $bdt = $this->curlHandler($brand['path']);
            if(!is_array($bdt)) {
                $this->log( 'ERROR', false, 'Неверный токен доступа' );
            } elseif(isset($bdt['error'])) {
                $this->log( 'ERROR', false, $bdt['error']['message'] );
            }
            $bdt['id'] = (int)$bdt['id'] + 100000;
            $cnt = 0;
            if(isset($bdt['categories'])) {
                foreach($bdt['categories'] as &$cat) {
                    $cat['products'] = $this->getComplects($cat['id']);
                    if(isset($cat['products'])) {
                        $cnt += count($cat['products']);
                    }
                }
                if($cnt) {
                    $this->data[ $brand['idx'] ] = $bdt;
                }
            }
            $this->total_prods += $cnt;
            $percent = round( 100.0 * (float)(++$bcnt)/(float)$total );
            $this->log( 'LOAD', $percent, 'Бренд ' . $brand['name'] . ' загружен. Пожалуйста, подождите. Загрузка бренда занимает до 2-х минут.' );
        }
        $dfc = '<?php return ' . var_export($this->data, true) . ';' . PHP_EOL;
        file_put_contents(DATAFILE, $dfc);
    }

    public function getPairs(){
        $this->pairs = $this->curlHandler('https://api.hiconix.ru/api/v2/pairs');
    }

    private function getPairsProd(){
        $count = 0;
        $pairs_ids = array();
        foreach ($this->pairs as $item){
            $count += count($item['outdoors']) * count($item['indoors']);
            $pairs_ids = array_merge($pairs_ids, $item['outdoors']);
            $pairs_ids = array_merge($pairs_ids, $item['indoors']);
        }
        $pairs_ids = array_unique($pairs_ids);
        $ids = array();
        foreach ($pairs_ids as $id){
            $ids[] = $id;
            if(count($ids) == 15){
                $prods = $this->getProductsByIdApi($ids);
                foreach ($prods as $item){
                    $this->pairs_prod[$item['id']] = $item;
                }
                $ids = array();
            }
        }
        if(count($ids) > 0){
            $prods = $this->getProductsByIdApi($ids);
            foreach ($prods as $item){
                $this->pairs_prod[$item['id']] = $item;
            }
        }
    }

    private function getProductsByIdApi($arr){
        foreach($arr as $item) {
            $req[] = 'id[]=' . $item;
        }
        $req = implode('&', $req);
        return $this->curlHandler('https://api.hiconix.ru/api/v2/products?' . $req);
    }

    private function getComplects($id_cat){
        $products = $this->curlHandler('https://api.hiconix.ru/api/v2/categories?id[]=' . $id_cat);
        $products = $products[0]['products'];
        $complect_prods = array();
        foreach ($products as $prod){
            $pairs = $this->curlHandler('https://api.hiconix.ru/api/v2/pairs?id=' . $prod['id']);
            if($pairs) {
                foreach ($pairs as $pair) {
                    $complect_prods = array_merge($complect_prods, $this->pairHandler($pair));
                }
            }
        }
        return $complect_prods;
    }

    // обработка комплекта
    private function pairHandler($pair){
        $complect_prods = array();
        $prefixname = $this->config['prefixname'];
        foreach ($pair['outdoors'] as $outdoors_id) {
            foreach ($pair['indoors'] as $indoors_id) {
                if(!in_array(strval($indoors_id) . strval($outdoors_id), $this->addedPairs)){
                    $this->addedPairs[] = strval($indoors_id) . strval($outdoors_id);
                    $outdoors = $this->pairs_prod[$outdoors_id];
                    $indoors = $this->pairs_prod[$indoors_id];
                    $prod = $indoors;

                    $prod['name'] = $indoors['name'] . ' / ' . $outdoors['name'];
                    if(!empty(trim($prefixname))){
                        if(strpos($prefixname, '{name}') !== false) {
                            $prod['name'] = str_replace('{name}', $prod['name'], $prefixname);
                        } else {
                            $prod['name'] = $prefixname . $prod['name'];
                        }
                    }

                    $prod['slug'] = $indoors['slug'] . '-' . $outdoors['slug'];
                    $prod['images'] = array_merge($indoors['images'], $outdoors['images']);
                    foreach ($prod['parameters'] as &$param){
                        if($param['param'] == 'price') {
                            $param['value'] = strval($this->getRecomPrice($indoors) + $this->getRecomPrice($outdoors));
                        }
                        if($param['param'] == 'noise') {
                            $param['param'] = 'noise_indoor';
                            $param['description'] = 'Уровень шума внутреннего блока';
                        }
                        if($param['param'] == 'weight') {
                            $param['param'] = 'weight_indoor';
                            $param['description'] = 'Вес внутреннего блока';
                        }
                    }

                    foreach ($outdoors['parameters'] as &$param){
                        if($param['param'] == 'noise') {
                            $param['param'] = 'noise_outdoor';
                            $param['description'] = 'Уровень шума наружного блока';
                            $prod['parameters'][] = $param;
                        }
                        if($param['param'] == 'weight') {
                            $param['param'] = 'weight_outdoor';
                            $param['description'] = 'Вес наружного блока';
                            $prod['parameters'][] = $param;
                        }
                    }

                    $tmp = $this->getWidthHeightLength($indoors);
                    $prod['parameters'][] = [
                        'description' => 'Размеры внутреннего блока (ШхВхД)',
                        'param' => 'widthheightlength_indoor',
                        'value' => $tmp['values'],
                        'units' => $tmp['units'],
                    ];

                    $tmp = $this->getWidthHeightLength($outdoors);
                    $prod['parameters'][] = [
                        'description' => 'Размеры наружного блока (ШхВхД)',
                        'param' => 'widthheightlength_outdoor',
                        'value' => $tmp['values'],
                        'units' => $tmp['units'],
                    ];

                    foreach ($prod['parameters'] as &$param){
                        if($param['param'] == 'noise' || $param['param'] == 'length' || $param['param'] == 'width'
                            || $param['param'] == 'height'){
                            $param = array();
                        }
                    }
                    $complect_prods[] = $prod;
                }
            }
        }
        return $complect_prods;
    }

    private function getRecomPrice($item){
        foreach ($item['parameters'] as $param){
            if($param['param'] == 'price') {
                return intval($param['value']);
            }
        }
        return 0;
    }

    private function getWidthHeightLength($item){
        $values = [];
        $units = [];
        foreach ($item['parameters'] as $param){
            if($param['param'] == 'width') {
                $values[0] = $param['value'];
                $units[0] = $param['units'];
            }
            if($param['param'] == 'height') {
                $values[1] = $param['value'];
                $units[1] = $param['units'];
            }
            if($param['param'] == 'length') {
                $values[2] = $param['value'];
                $units[2] = $param['units'];
            }
        }
        return [
            'values' => implode('x', $values),
            'units' => $units[0],
        ];
    }
}